"use strict";

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/* global include enigma */
var WebsyQlikObjectManagerLight =
/*#__PURE__*/
function () {
  function WebsyQlikObjectManagerLight(options) {
    _classCallCheck(this, WebsyQlikObjectManagerLight);

    var defaults = {
      helpEvent: 'mouseover',
      actions: []
    };
    this.app = null;
    this.paused = false;
    this.supportedChartTypes = [];
    this.activeViews = [];
    this.chartLibrary = {};
    this.globalObjectsLoaded = false;

    if (options.visualisationPlugins && options.visualisationPlugins.length > 0) {
      for (var i = 0; i < options.visualisationPlugins.length; i++) {
        this.registerVisualisation(options.visualisationPlugins[i].id, options.visualisationPlugins[i].definition);
      }
    }

    this.options = _extends({}, defaults, options);
  }

  _createClass(WebsyQlikObjectManagerLight, [{
    key: "buildChildElement",
    value: function buildChildElement(elementId, text) {
      var html = "\n      <article id='".concat(elementId, "_vis' class='websy-vis-article'></article>\n      <div id='").concat(elementId, "_loading' class='websy-loading-container'><div class='websy-ripple'><div></div><div></div></div></div>\n    ");

      if (text && text !== '') {
        html += "\n        <i class='websy-vis-help-listener' data-element='".concat(elementId, "'></i>\n        <div id='").concat(elementId, "_help' class='websy-vis-help'><span>").concat(text || '', "</span></div>        \n      ");
      }

      return html;
    }
  }, {
    key: "init",
    value: function init() {
      var _this = this;

      return new Promise(function (resolve, reject) {
        _this.prep();

        _this.connectToApp().then(resolve, reject);
      });
    }
  }, {
    key: "pause",
    value: function pause() {
      this.paused = true;
    }
  }, {
    key: "play",
    value: function play(resume) {
      this.paused = false;

      if (resume === true) {
        this.loadObjects('global');

        for (var i = 0; i < this.activeViews.length; i++) {
          this.loadObjects(this.activeViews[i]);
        }
      }
    }
  }, {
    key: "prep",
    value: function prep() {
      var _this2 = this;

      for (var view in this.options.views) {
        // sort out the elements in each view
        for (var o = 0; o < this.options.views[view].objects.length; o++) {
          var config = this.options.views[view].objects[o];
          var el = document.getElementById(config.elementId);

          if (el) {
            el.innerHTML += this.buildChildElement(config.elementId, config.help);

            if (config.help && config.help !== '') {
              el.addEventListener(this.options.helpEvent, this.handleEvent.bind(this));
              el.addEventListener('mouseout', this.handleEvent.bind(this));
            }
          }
        }
      } // setup  the event listeners for the actions
      // actions should not have a visualisation in the same property structure


      var _loop = function _loop(a) {
        var el = document.getElementById(_this2.options.actions[a].elementId);

        if (el) {
          el.addEventListener(_this2.options.actions[a].event, function () {
            var _loop2 = function _loop2(i) {
              var item = _this2.options.actions[a].items[i];

              if (item.field) {
                _this2.app.getField(item.field).then(function (field) {
                  field[item.method].apply(field, _toConsumableArray(item.params));
                });
              } else {
                var _this2$app;

                (_this2$app = _this2.app)[item.method].apply(_this2$app, _toConsumableArray(item.params));
              }
            };

            for (var i = 0; i < _this2.options.actions[a].items.length; i++) {
              _loop2(i);
            }
          });
        }
      };

      for (var a = 0; a < this.options.actions.length; a++) {
        _loop(a);
      }
    }
  }, {
    key: "connectToApp",
    value: function connectToApp() {
      var _this3 = this;

      return new Promise(function (resolve, reject) {
        // check for enigma.js      
        var originalId = _this3.options.enigmaConfig.app;
        _this3.options.enigmaConfig.app = _this3.normalizeId(_this3.options.enigmaConfig.app);

        if (typeof enigma === 'undefined') {
          reject({
            error: 'Enigma.js not found.'
          });
          return;
        }

        if (typeof _this3.options.enigmaSchema === 'undefined') {
          reject({
            error: 'enigmaSchema property not found.'
          });
          return;
        }

        var config = {
          schema: _this3.options.enigmaSchema,
          url: _this3.options.enigmaConfig.url
        };
        var session = enigma.create(config);
        _this3.session = session;
        session.open().then(function (global) {
          global.getActiveDoc().then(function (app) {
            if (app) {
              return app;
            } else {
              return global.openDoc(originalId);
            }
          }, function (err) {
            console.log(err);
            return global.openDoc(originalId);
          }).then(function (app) {
            _this3.app = app;

            if (_this3.options.views.global) {
              _this3.executeActions('global').then(function () {
                // this.loadObjects('global')
                resolve();
              });
            } else {
              resolve();
            }
          }, function (err) {
            reject(err);
          });
        }, function (err) {
          reject(err);
        });
        session.on('traffic:received', function (data) {
          if (typeof data.suspend !== 'undefined') {
            _this3.sessionSuspended();
          }
        });
        session.on('notification:*', function (eventName, data) {
          if (eventName === 'OnAuthenticationInformation') {
            if (data.mustAuthenticate === true && _this3.options.enigmaConfig.authUrl) {
              window.location = _this3.options.enigmaConfig.authUrl;
            } else if (data.mustAuthenticate === false) {
              _this3.user = {
                userDirectory: data.userDirectory,
                userId: data.userId
              };
            }
          } else {
            _this3.sessionOnNotification(data);
          }
        });
        session.on('suspended', _this3.sessionSuspended.bind(_this3));
        session.on('resumed', _this3.sessionResumed.bind(_this3));
      });
    }
  }, {
    key: "loadView",
    value: function loadView(view) {
      var _this4 = this;

      if (view === '') {
        return;
      }

      if (this.activeViews.indexOf(view) === -1 && view !== 'global') {
        this.activeViews.push(view);
      }

      this.executeActions(view).then(function () {
        if (_this4.globalObjectsLoaded === false) {
          _this4.loadObjects('global');

          _this4.globalObjectsLoaded = true;
        }

        _this4.loadObjects(view);
      });
    } // executeActions(view){
    //   return new Promise((resolve, reject) => {
    //     if (!this.options.views[view].actions) {
    //       resolve()
    //     }
    //     let actionList = []
    //     for (var i = 0; i < this.options.views[view].actions.length; i++) {
    //       let item = this.options.views[view].actions[i]
    //       if (item.field) {
    //         actionList.push(this.app.getField(item.field).then(field=>{
    //           field[item.method](...item.params)
    //         }))
    //       }
    //       else {
    //         actionList.push(this.app[item.method](...item.params))
    //       }        
    //     }
    //     Promise.all(actionList).then(resolve, reject)
    //   })
    // }

  }, {
    key: "executeAction",
    value: function executeAction(index, actionList, callbackFn) {
      var _this5 = this;

      var item = actionList[index];

      if (item.field) {
        this.app.getField(item.field).then(function (field) {
          field[item.method].apply(field, _toConsumableArray(item.params)).then(function () {
            if (item.lock === true) {
              field.lock().then(function () {
                index++;

                if (index === actionList.length) {
                  callbackFn();
                } else {
                  _this5.executeAction(index, actionList, callbackFn);
                }
              });
            } else {
              index++;

              if (index === actionList.length) {
                callbackFn();
              } else {
                _this5.executeAction(index, actionList, callbackFn);
              }
            }
          });
        });
      } else {
        var _this$app;

        (_this$app = this.app)[item.method].apply(_this$app, _toConsumableArray(item.params)).then(function () {
          index++;

          if (index === actionList.length) {
            callbackFn();
          } else {
            _this5.executeAction(index, actionList, callbackFn);
          }
        });
      }
    }
  }, {
    key: "executeActions",
    value: function executeActions(view) {
      var _this6 = this;

      return new Promise(function (resolve, reject) {
        if (!_this6.options.views[view].actions || _this6.options.views[view].actions.length === 0) {
          resolve();
        }

        _this6.executeAction(0, _this6.options.views[view].actions, resolve);
      });
    }
  }, {
    key: "loadObjects",
    value: function loadObjects(view) {
      if (this.paused === true) {
        return;
      }

      var objList = this.options.views[view].objects;

      if (objList && objList.length > 0) {
        for (var i = 0; i < objList.length; i++) {
          if (objList[i].objectId) {
            objList[i].attached = true;

            if (objList[i].vis && objList[i].vis.render) {
              objList[i].vis.render();
            } else if (objList[i].vis.render) {
              objList[i].render();
            }
          } else if (objList[i].definition) {
            this.createObjectFromDefinition(objList[i]);
          }
        }
      }
    }
  }, {
    key: "closeObjects",
    value: function closeObjects(view) {
      if (view === '') {
        return;
      }

      var viewIsActive = this.activeViews.indexOf(view);

      if (viewIsActive !== -1) {
        this.activeViews.splice(viewIsActive, 1);
      }

      var objList = this.options.views[view].objects;

      if (objList && objList.length > 0) {
        for (var i = 0; i < objList.length; i++) {
          if (objList[i].vis) {
            objList[i].attached = false;

            if (objList[i].vis.close) {
              objList[i].vis.close();
            }
          } else if (objList[i].objectId) {
            if (objList[i].close) {
              objList[i].close();
            }

            this.destroyObjectFromId(objList[i]);
          }
        }
      }
    }
  }, {
    key: "handleEvent",
    value: function handleEvent(event) {
      if (event.target.classList.contains('websy-vis-help-listener')) {
        var elementId = event.target.attributes['data-element'];

        if (elementId.value) {
          this.toggleHelp("".concat(elementId.value, "_help"));
        }
      }
    }
  }, {
    key: "createObjectFromDefinition",
    value: function createObjectFromDefinition(objectConfig) {
      var _this7 = this;

      /* global objectConfig */
      this.app.createSessionObject(objectConfig.definition).then(function (model) {
        objectConfig.objectId = model.id;
        objectConfig.attached = true;

        if (_this7.supportedChartTypes.indexOf(objectConfig.definition.qInfo.qType) !== -1) {
          objectConfig.vis = new _this7.chartLibrary[objectConfig.definition.qInfo.qType]("".concat(objectConfig.elementId, "_vis"), {
            model: model,
            def: objectConfig.definition,
            app: _this7.app
          });
          model.on('changed', function () {
            if (objectConfig.attached === true && _this7.paused === false) {
              objectConfig.vis.render();
            }
          });
        } else if (objectConfig.render && typeof objectConfig.render === 'function') {
          objectConfig.vis = {};
          objectConfig.render(objectConfig, model);
          model.on('changed', function () {
            if (objectConfig.attached === true && _this7.paused === false) {
              objectConfig.render(objectConfig, model);
            }
          });
        }
      });
    }
  }, {
    key: "destroyObjectFromId",
    value: function destroyObjectFromId(objectConfig) {
      var hostEl = document.getElementById("".concat(objectConfig.elementId, "_vis"));

      if (hostEl) {
        hostEl.innerHTML = '';
      }

      this.app.destroySessionObject(objectConfig.objectId);
    }
  }, {
    key: "detachObject",
    value: function detachObject(objectConfig) {
      objectConfig.attached = false;
    }
  }, {
    key: "normalizeId",
    value: function normalizeId(id) {
      return id.replace(/\s:\\\//, '-');
    }
  }, {
    key: "sessionOnNotification",
    value: function sessionOnNotification(event) {
      console.log('Notification');
      console.log(event);
    }
  }, {
    key: "sessionOnTraffic",
    value: function sessionOnTraffic(event) {
      console.log('Traffic');
      console.log(event);
    }
  }, {
    key: "sessionResumed",
    value: function sessionResumed(event) {
      console.log('Session Resumed');
      console.log(event);
    }
  }, {
    key: "sessionSuspended",
    value: function sessionSuspended(event) {
      console.log('Session suspended');
      console.log(event);
    }
  }, {
    key: "showHelp",
    value: function showHelp(elementId) {
      var el = document.getElementById(elementId);

      if (el) {
        el.classList.add('active');
      }
    }
  }, {
    key: "hideHelp",
    value: function hideHelp(elementId) {
      var el = document.getElementById(elementId);

      if (el) {
        el.classList.remove('active');
      }
    }
  }, {
    key: "toggleHelp",
    value: function toggleHelp(elementId) {
      var el = document.getElementById(elementId);

      if (el) {
        el.classList.toggle('active');
      }
    }
  }, {
    key: "onError",
    value: function onError(err) {
      console.log(err);
    }
  }, {
    key: "onClose",
    value: function onClose(msg) {}
  }, {
    key: "resize",
    value: function resize() {
      for (var i = 0; i < this.activeViews.length; i++) {
        this.resizeObjects(this.activeViews[i]);
      }
    }
  }, {
    key: "resizeObjects",
    value: function resizeObjects(view) {
      if (view === '') {
        return;
      }

      var objList = this.options.views[view].objects;

      if (objList && objList.length > 0) {
        for (var i = 0; i < objList.length; i++) {
          if (objList[i].objectId) {
            if (objList[i].vis && objList[i].vis.resize) {
              objList[i].vis.resize();
            } else if (objList[i].resize) {
              objList[i].resize();
            }
          }
        }
      }
    }
  }, {
    key: "registerVisualisation",
    value: function registerVisualisation(name, classDef) {
      if (name.indexOf(/\s/) !== -1) {
        console.log('Failed to register Chart Extension. Chart name must not contain spaces.');
        return;
      }

      if (this.supportedChartTypes.indexOf(name) !== -1) {
        console.log('Failed to register Chart Extension. Chart name already exists.');
        return;
      }

      this.supportedChartTypes.push(name);
      this.chartLibrary[name] = classDef;
    }
  }], [{
    key: "mergeObjects",
    value: function mergeObjects() {
      // Variables
      var extended = {};
      var deep = false;
      var i = 0; // Check if a deep merge

      if (typeof arguments[0] === 'boolean') {
        deep = arguments[0];
        i++;
      } // Merge the object into the extended object


      var merge = function merge(obj) {
        for (var prop in obj) {
          if (obj.hasOwnProperty(prop)) {
            if (deep && Object.prototype.toString.call(obj[prop]) === '[object Object]') {
              // If we're doing a deep merge and the property is an object
              extended[prop] = this.mergeObjects(true, extended[prop], obj[prop]);
            } else {
              // Otherwise, do a regular merge
              if (Array.isArray(extended[prop]) && Array.isArray(obj[prop])) {
                extended[prop] = extended[prop].concat(obj[prop]);
              } else {
                extended[prop] = obj[prop];
              }
            }
          }
        }
      }; // Loop through each object and conduct a merge


      for (; i < arguments.length; i++) {
        merge(arguments[i]);
      }

      return extended;
    }
  }]);

  return WebsyQlikObjectManagerLight;
}();
